package com.yutou.controller.Web;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.yutou.utlis.*;
import org.json.JSONObject;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSON;
import com.yutou.enums.ModEnum;
import com.yutou.enums.UserEnum;
import com.yutou.mybatis.TMod;
import com.yutou.mybatis.UUserdata;
import com.yutou.mybatis.User;
import com.yutou.services.ModService;
import com.yutou.services.UserService;

@Controller
@RequestMapping("/user")
public class WebUserController {
	@Resource
	UserService userService;
	@Resource
	ModService modService;

	@RequestMapping("/mods.do")
	@ResponseBody
	public String getModOfUser(HttpServletRequest request) {
		JSONObject json = new JSONObject();
		String ujson = RedisUtlis.get(Tools.getCookie(request,"login"));
		JSONObject uujson=new JSONObject(ujson);
		int uid = Integer.parseInt(uujson.getString("uid"));
		List<TMod> list = modService.getUserOfId(uid);
		if (list != null && list.size() > 0) {
			json.put("code", ModEnum.SUCCESS.code);
			json.put("msg", ModEnum.SUCCESS.msg);
			json.put("data", JSON.toJSON(list));
		} else {
			json.put("code", ModEnum.ERROR.code);
			json.put("msg", ModEnum.ERROR.msg);
			json.put("data", "[]");
		}
		return json.toString();
	}

	@RequestMapping("/udata.do")
	@ResponseBody
	public String getUserData(HttpServletRequest request) {
		JSONObject userdata = new JSONObject();
		try {
			String ujson = RedisUtlis.get(Tools.getCookie(request,"login"));
			if(ujson==null){
				userdata.put("code", UserEnum.LOGIN_NOT_LOGIN.code);
				userdata.put("msg", UserEnum.LOGIN_NOT_LOGIN.msg);
			}else {
				JSONObject json = new JSONObject(ujson);
				int uid = json.getInt("uid");
				userdata.put("msg", UserEnum.LOGIN_SUCCESS.code);
				userdata.put("message", "ok");
				userdata.put("data", userService.getUserData(uid).getJSONObject("data"));
			}
		} catch (Exception e) {
			// TODO: handle exception
			userdata.put("code", UserEnum.LOGIN_NOT_LOGIN.code);
			userdata.put("msg", UserEnum.LOGIN_NOT_LOGIN.msg);
		}
		return userdata.toString();
	}

	@RequestMapping("/updateUserData.do")
	@ResponseBody
	public String setUserData(HttpServletRequest request, String email, String uname) {
		JSONObject userdata = new JSONObject();
		try {
			if (!Tools.StringIsEmpty(email) && !Pattern.matches(
					"^([a-z0-9A-Z]+[-|\\.]?)+[a-z0-9A-Z]@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-zA-Z]{2,}$", email)) {
				userdata.put("code", -1);
				userdata.put("msg", "邮箱填写错误");
				return userdata.toString();
			}
			JSONObject imageJson = (JSONObject) Tools.getSessionKey(request, "image");
			String ujson = RedisUtlis.get(Tools.getCookie(request,"login"));
			JSONObject json=new JSONObject(ujson);
			int uid =  json.getInt("uid");
			User user = userService.getUser(uid);
			UUserdata data = userService.getUUserData(uid);
			if (!Tools.StringIsEmpty(email))
				data.setEmail(email);
			if (imageJson != null && !imageJson.isNull("url")) {
				data.setImage(imageJson.getString("url"));
				File lock = new File(imageJson.getString("path"));
				if (lock.exists()) {
					lock.delete();
				}
			}
			if (!Tools.StringIsEmpty(uname)) {
				user.setUname(uname);
			}
			userService.updateUser(user);
			userService.updateUserData(data);

			userdata.put("code", 100);
			userdata.put("msg", "更新成功");
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
		}
		return userdata.toString();
	}

	@RequestMapping("/updata_image.do")
	@ResponseBody
	public String serUserData_Image(@RequestParam(value = "image", required = false) MultipartFile file,
			HttpServletRequest request) {
		JSONObject json = new JSONObject();
		try {
			String ujson =RedisUtlis.get(Tools.getCookie(request,"login"));
			JSONObject reJson=new JSONObject(ujson);
			int uid = Integer.parseInt(reJson.getString("uid"));
			String url = ServiceStaticValue.SERVICE_URL
					+ Tools.createFile(request, "user/icon/" + uid + "/", file, "image.png");
			json.put("id", 1);
			json.put("url", url);
			File lock = new File(Tools.getRootPath(request) + "/user/icon/" + uid + "/lock");
			lock.createNewFile();
			json.put("path", lock.getAbsolutePath());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Tools.saveSession(request, "image", json);
		return json.toString();
	}
	
	@RequestMapping("/repassword.do")
	@ResponseBody
	public String rePassword(String uid,String name,HttpServletRequest request) {
		JSONObject json=new JSONObject();
		try {
			String ujson =RedisUtlis.get(Tools.getCookie(request,"login"));
			JSONObject reJson=new JSONObject(ujson);
			Log.i(ujson);
			int uid_ =reJson.getInt("uid");
			if(!Tools.StringIsEmpty(uid)) {
				try {
					uid_=Integer.parseInt(uid);
				}catch (Exception e) {
					// TODO: handle exception
				}
			}
			if(!Tools.StringIsEmpty(name)) {
				try {
					uid_=userService.getUserId(name);
				} catch (Exception e) {
					// TODO: handle exception
				}
			}
			User user=userService.getUser(uid_);
			UUserdata userdata=userService.getUUserData(user.getId());
			if(Tools.StringIsEmpty(userdata.getEmail())) {
				json.put("msg", "邮箱未填写");
				json.put("code", -1);
			}else {
				String uuid=UUID.randomUUID().toString();
				RedisUtlis.set(uuid, user.getId()+"", 1800);
			EmailTools.send(userdata.getEmail(), "[魔改宝盒]用户密码重置邮件", "您于 "+new SimpleDateFormat("yyyy年MM月DD日 HH时mm分ss秒").format(new Date())+""
					+ "申请了重置密码，如本次操作并非您本人操作请无视本邮件。<br>重置邮件链接： "+ServiceStaticValue.SERVICE_URL+"user/security/reset.do?sign="+uuid+""
							+ " <br>该链接30分钟内有效");
			json.put("msg", "邮件已发送");
			json.put("code", 0);
			}
		}catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			json.put("msg", "异常状态");
			json.put("code", -1);
		}
		return json.toString();
	}
	@RequestMapping("/security/reset.do")
	@ResponseBody
	public String reset(String sign,HttpServletRequest request,HttpServletResponse response) {
		try {
			Tools.saveSession(request, "uuid", sign);
			response.sendRedirect("/html/resetPassword.html");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "自动跳转失败，<a href='/html/resetPassword.html'>请点击这里手动跳转</a>";
		}
		return "跳转中";
	}
	
	@RequestMapping("/resetPassword.do")
	@ResponseBody
	public String resetPassword(HttpServletRequest request) {
		JSONObject json=new JSONObject();
		try {
			String uuid=Tools.loadSession(request, "uuid");

			String uid = RedisUtlis.get(uuid);
			if(Tools.StringIsEmpty(uid)) {
				json.put("msg", "非法操作，已记录您的IP");
				json.put("code", -1);
			}else {
				User user=userService.getUser(Integer.parseInt(uid));
				String password=request.getParameter("password");
				user.setPass(password);
				RedisUtlis.removeLoginState(uid);
				RedisUtlis.remove(uuid);
				userService.updateUser(user);
				json.put("msg", "密码修改成功");
				json.put("code", 0);
			}
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			json.put("msg", "异常状态");
			json.put("code", -1);
		}
		return json.toString();
	}
}
