package com.yutou.controller.admin;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.yutou.utlis.Log;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.yutou.enums.POWER;
import com.yutou.mybatis.UUserdata;
import com.yutou.mybatis.User;
import com.yutou.services.PermissionService;
import com.yutou.services.UserService;
import com.yutou.utlis.Tools;

@Controller
@RequestMapping("/admin/user/")
public class AdminUserController {
	@Resource
	UserService userService;
	@Resource
	PermissionService powerService;

	@RequestMapping("/list.do")
	@ResponseBody
	public String list(String type,String key,String page,String limit,HttpServletRequest request,HttpServletResponse response) {
		Tools.resetCode(response);
		JSONObject json=new JSONObject();
		JSONArray array=new JSONArray();
		if(!isAdmin(request)) {
			json.put("code", -1);
			json.put("msg", "非法操作");
			json.put("data", array);
			return json.toString();
		}
		if(!Tools.StringIsEmpty(type,key)) {
			return searchUser(type, key, request, response);
		}
		List<User> list=userService.getUserList(Integer.parseInt(page), Integer.parseInt(limit));
		for (User u : list) {
			try {
				JSONObject tmp=new JSONObject();
				tmp.put("uid", u.getId());
				tmp.put("name", u.getName());
				tmp.put("uname",u.getUname());
				UUserdata data=userService.getUUserData(u.getId());
				tmp.put("benTime", data.getBeantime()==null?"无记录":new SimpleDateFormat("yyyy/MM/DD HH:mm:ss").format(data.getBeantime()));
				tmp.put("unBenTime",data.getUnbeantime()==null?"无记录": new SimpleDateFormat("yyyy/MM/DD HH:mm:ss").format(data.getUnbeantime()));
				tmp.put("benIndex", data.getBeanindex());
				array.put(tmp);
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();
			}
			
		}
		json.put("code", 0);
		json.put("msg", "成功");
		json.put("count", userService.getUserList(-1, -1).size());
		json.put("data", array);
		return json.toString();
	}
	@RequestMapping("/ben.do")
	@ResponseBody
	public String ben(String uid,String time,HttpServletRequest request,HttpServletResponse response) {
		Tools.resetCode(response);
		JSONObject json=new JSONObject();
		if(!isAdmin(request)) {
			json.put("code", -1);
			json.put("msg", "非法操作");
			return json.toString();
		}
		try {
			Date date=new SimpleDateFormat("yyyy-MM-DD HH:mm:ss").parse(time);
			UUserdata user=userService.getUUserData(Integer.valueOf(uid));
			user.setBeantime(new Date());
			user.setUnbeantime(date);
			try {
				user.setBeanindex(user.getBeanindex()+1);
			} catch (Exception e) {
				// TODO: handle exception
				user.setBeanindex(1);
			}
			
			userService.updateUserData(user);
			json.put("code", 0);
			json.put("msg", "操作成功");
			return json.toString();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			json.put("code", -1);
			json.put("msg", "系统内部错误");
			return json.toString();
		}
	}
	@RequestMapping("/unben.do")
	@ResponseBody
	public String unBen(String uid,HttpServletRequest request,HttpServletResponse response) {
		Tools.resetCode(response);
		JSONObject json=new JSONObject();
		if(!isAdmin(request)) {
			json.put("code", -1);
			json.put("msg", "非法操作");
			return json.toString();
		}
		UUserdata user=userService.getUUserData(Integer.valueOf(uid));
		user.setUnbeantime(new Date());
		userService.updateUserData(user);
		json.put("code", 0);
		json.put("msg", "操作成功");
		return json.toString();
	}
	
	public String searchUser(String type,String key,HttpServletRequest request,HttpServletResponse response) {
		Tools.resetCode(response);
		JSONObject json=new JSONObject();
		JSONArray array=new JSONArray();
		if(!isAdmin(request)) {
			json.put("code", -1);
			json.put("msg", "非法操作");
			json.put("data", array);
			return json.toString();
		}
		if(Tools.StringIsEmpty(key)) {
			json.put("code", -1);
			json.put("msg", "关键字为空");
			json.put("data", array);
			return json.toString();
		}
		try {
			List<User> list=userService.searchUser(type, key);
			for (User u : list) {
				try {
					JSONObject tmp=new JSONObject();
					tmp.put("uid", u.getId());
					tmp.put("name", u.getName());
					tmp.put("uname",u.getUname());
					UUserdata data=userService.getUUserData(u.getId());
					tmp.put("benTime", data.getBeantime()==null?"无记录":new SimpleDateFormat("yyyy/MM/DD HH:mm:ss").format(data.getBeantime()));
					tmp.put("unBenTime",data.getUnbeantime()==null?"无记录": new SimpleDateFormat("yyyy/MM/DD HH:mm:ss").format(data.getUnbeantime()));
					tmp.put("benIndex", data.getBeanindex());
					array.put(tmp);
				} catch (Exception e) {
					// TODO: handle exception
					e.printStackTrace();
				}
			}
			json.put("code", 0);
			json.put("msg", "成功");
			json.put("count", userService.getUserList(-1, -1).size());
			json.put("data", array);
			return json.toString();
		}catch (Exception e) {
			// TODO: handle exception
			json.put("code", -1);
			json.put("msg", "意外错误");
			json.put("data", array);
			return json.toString();
		}
	}
	
	public boolean isAdmin(HttpServletRequest request) {
		int uid=Tools.getUserId(request);
		User user=userService.getUser(uid);
		Log.i(user);
		if(user.getPower()<powerService.getGroup(POWER.ADMIN,null).getPermission()) {
			return false;
		}
		return true;
	}
}
