package com.yutou.services;

import java.util.List;

import org.json.JSONObject;

import com.yutou.mybatis.TMod;
import com.yutou.mybatis.TModcomment;
import com.yutou.mybatis.TModfile;
import com.yutou.mybatis.TModfilePath;
import com.yutou.mybatis.TModpower;
import com.yutou.mybatis.TModtag;

public interface ModService {
	/**
	 * 上传mod
	 * @param json
	 * @return 0失败，1成功
	 */
	int upload(JSONObject json);
	/**
	 * 获取mod基础信息
	 * @param mid mod的id
	 * @return 
	 */
	TMod getMod(int mid, int upower);
	/**
	 * 搜索mod
	 * @param search 搜索关键词
	 * @return mod列表
	 */
	List<TMod> getMod(String search, int upower);
	/**
	 * 获取mod的评论
	 * @param mid mod的id
	 * @return 评论
	 */
	List<TModcomment> getModComment(int mid);
	/**
	 * 获取mod的下载数
	 * @param mid mod的id
	 * @return
	 */
	int getModDownload(int mid);
	/**
	 * 获取mod的下载地址
	 * @param mid mod的id #注意,下载地址是没有包含主URL的
	 * @return
	 */
	List<TModfile> getModFile(int mid, int uid);
	/**
	 * 获取mod应该安装到的路径
	 * @param mid mod的id
	 * @return
	 */
	String getPath(int mid);
	/**
	 * 获取所有安装路径
	 * @return 集合
	 */
	List<TModfilePath> getInstallPaths();
	/**
	 * 获取指定范围的mod的集合 #min或max为-1时，表示全部
	 * @param min 最小数
	 * @param max 最大数
	 * @return
	 */
	List<TMod> getMods(int min, int max, int upower);
	/**
	 * 获取mod的权限
	 * @param mid mod的id
	 * @return
	 */
	TModpower getModPower(int mid);
	/**
	 * 获取所有Mod类型的mod
	 * @param tid
	 * @return
	 */
	List<TMod> getTypeOfMod(int tid);
	/**
	 * 获取所有舰娘类型的MOD
	 * @param tid
	 * @return
	 */
	List<TMod> getTypeOfJian(int tid);
	/**
	 * 获取该用户所上传的mod
	 * @param uid
	 * @return
	 */
	List<TMod> getUserOfId(int uid);
	/**
	 * 获取相关mod
	 * @param mid
	 * @param rmid
	 * @return
	 */
	List<TMod> getReMod(int mid, int rmid);
	
	List<TModtag> getModTag(int mid);
	/**
	 * user add Conllection
	 * @param uid user id
	 * @param mid mod id
	 * @return 0 error, 1 success, -1 user already Conllection
	 */
	int setConllection(String uid, String mid);
	/**
	 * 设置mod权限
	 * @param mid
	 * @param power
	 * @return
	 */
	int setModPower(int mid, int power);
	/**
	 * 移动mod
	 * @param mid
	 * @param jType
	 * @param mType
	 * @return
	 */
	int moveMod(int mid, String jType, String mType);
	/**
	 * 获取MOD类型和舰娘类型的信息
	 * @return
	 */
	JSONObject getTypeInfo();
}
