package com.yutou.utlis;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Date;
import java.util.Enumeration;
import java.util.Random;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSON;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.web.multipart.MultipartFile;

import com.yutou.mybatis.UUserdata;


public class Tools {

	/**
	 * 保存上传的文件
	 * 
	 * @param request
	 * @param path
	 *            路径
	 * @param file
	 *            文件
	 * @return
	 * @throws Exception
	 */
	public static String createFile(HttpServletRequest request, String path, MultipartFile file,String type) throws Exception {
		String savePath =new File("").getAbsolutePath()+File.separator +"html"+File.separator+  path;
	//	savePath=savePath.replace("/", "\\");
		File servicePath = new File(savePath);
		if (!servicePath.exists()) {
			servicePath.mkdirs();
		}
		String fileName=file.getOriginalFilename();
		if(type!=null) {
			fileName=type;
		}
		File saveFile = new File(savePath + "/" + fileName);
		if(saveFile.exists()) {
			if(!saveFile.delete()) {
				saveFile = new File(savePath + "/" + fileName.replace(".", "_"+new Date().getTime()+"."));
			}
		}
		file.transferTo(saveFile);
		//Ossupload oss = new Ossupload();
		//String url = oss.upload(saveFile, path);
		//oss.close();
		fileName=URLEncoder.encode(fileName,"UTF-8");
		Log.i("上传文件保存路径："+saveFile.getAbsolutePath());
		return path+fileName;
	}
	/**
	 * 获取项目路径
	 * @param request
	 * @return
	 */
	public static String getPath(HttpServletRequest request) {
		return request.getServletContext().getRealPath("/") + "/";
	}
	/**
	 * 获取客户端IP
	 * 
	 * @param request
	 * @return
	 */
	public static String getRemoteAddress(HttpServletRequest request) {
		String ip = request.getHeader("x-forwarded-for");
		if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
			ip = request.getHeader("WL-Proxy-Client-IP");
		}
		if (ip == null || ip.length() == 0 || ip.equalsIgnoreCase("unknown")) {
			ip = request.getRemoteAddr();
		}
		return ip;
	}

	/**
	 * 获取网络请求（GET请求）
	 * 
	 * @param url
	 *            地址
	 * @return 请求内容
	 */
	public static String getHTTPUrlString(String url) {
		try {
			Log.i("URL请求", url);
			HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
			BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream(), "UTF-8"));
			String tmp = reader.readLine();
			String json = "";
			while (tmp != null) {
				json += tmp;
				tmp = reader.readLine();
				Log.i(tmp);
			}
			Log.i("URL请求回调", json);
			return json;
		} catch (Exception e) {
			e.printStackTrace();
			Log.printf_Error(e, Tools.class);
		}
		return null;
	}

	/**
	 * 获取保存在Cookie里的登录信息
	 * 
	 * @param request
	 * @return json:{id/uname/session}
	 */

	public static JSONObject loadCookie(HttpServletRequest request) {
		Cookie[] cookies = request.getCookies();
		JSONObject json = new JSONObject();
		for (Cookie cookie : cookies) {
			Log.i("Cookie-Key:" + cookie.getName() + "\tCookie-Value:" + cookie.getValue());
			if (cookie.getName().equals("uname")) {
				json.put("uname", cookie.getValue()+"");
			}
			if (cookie.getName().equals("session")) {
				json.put("session", cookie.getValue()+"");
			}
		}
		if(json.toString().equals("{}")) {
			json=null;
		}
		return json;
	}

	/**
	 * 根据SessionId获取对应的value
	 * 
	 * @param request
	 * @param key
	 * @return
	 */
	public static Object getSessionKey(HttpServletRequest request, String key) {
		return request.getSession().getAttribute(key);
	}

	/**
	 * 判断字符串是否为空
	 * 
	 * @param string
	 * @return
	 */
	public static boolean StringIsEmpty(String... string) {
		for (String str : string) {
			if (str == null || "".equals(str) || str.trim().length() == 0) {
				return true;
			}
		}
		return false;

	}

	/**
	 * 保存登录信息到Cookie里去
	 * 
	 * @param request
	 * @param key
	 * @return sessionId
	 */
	public static String saveCookie(HttpServletRequest request, HttpServletResponse response, String key) {
		return setCookie(request, response, key,-1);
	}
	/**
	 * 设置Cookie
	 * @param request
	 * @param response
	 * @param key
	 * @param time 生命周期，为0时即为删除
	 * @return
	 */
	private static String setCookie(HttpServletRequest request, HttpServletResponse response, String key,int time) {
		Cookie name = new Cookie("uname", key);
		Cookie session = new Cookie("session", request.getSession().getId());
		if(time!=-1) {
			name.setMaxAge(time);
			session.setMaxAge(time);
		}
		response.addCookie(name);
		response.addCookie(session);
		return request.getSession().getId();
	}
	/**
	 *  设置Cookie
	 * @param response
	 * @param key
	 * @param value
	 * @param time
	 */
	public static void setCookie(HttpServletResponse response, String key,String value,int time) {
		Cookie cookie = new Cookie(key, value);
		if(time!=-1) {
			cookie.setMaxAge(time);
		}
		response.addCookie(cookie);
	}
	/**
	 * 获取Cookie
	 * @param request
	 * @param key
	 * @return
	 */
	public static String getCookie(HttpServletRequest request,String key) {
		Cookie[] cookies = request.getCookies();
		String value=null;
		for (Cookie cookie : cookies) {
			if (key!=null&&cookie.getName().equals(key)) {
				value=cookie.getValue();
				break;
			}else if(key==null){
				if(value==null){
					value="";
				}
				value+= JSON.toJSONString(cookie);
			}
		}
		return value;
	}
	/**
	 * 删除Cookie
	 * @param request
	 * @param response
	 * @param key
	 * @return
	 */
	public static String deleteCookie(HttpServletRequest request, HttpServletResponse response, String key) {
		return setCookie(request, response, key, 0);
	}
	/**
	 * 保存Key到Session
	 * 
	 * @param request
	 * @param key
	 * @param value
	 * @return
	 */
	public static String saveSession(HttpServletRequest request, String key, Object value) {
		request.getSession().setAttribute(key, value);
		return request.getSession().getId();
	}

	/**
	 * 获取Session，如果Key没保存在Session，则返回null
	 * 
	 * @param request
	 * @param key
	 * @return sessionId
	 */
	public static String loadSession(HttpServletRequest request, String key) {
		String sessionid;
		try {
			sessionid = request.getSession().getAttribute(key).toString();
		} catch (Exception e) {
			// TODO: handle exception
			sessionid = null;
		}
		/*
		 * String referer=request.getHeader("referer");//访问来源验证
		 * if(referer==null||!referer.equals("http://www.527youxi.com")) { return null;
		 * }
		 */
		return sessionid;
	}

	/**
	 * 移除Session中的Key
	 * 
	 * @param request
	 * @param key
	 * @return 成功/失败
	 */
	public static boolean removeSession(HttpServletRequest request, String key) {
		try {
			request.getSession().removeAttribute(key);

		} catch (Exception e) {
			// TODO: handle exception
			return false;
		}
		return true;
	}

	
	public static void main(String[] args) {
		JSONArray array = new JSONArray();
		array.put(1);
		array.put(2);
		array.put(3);
		array.put(4);
		array.put(5);
		Log.i(array.toString());
		Log.i(array.toString());
	}
	/**
	 * N以内的不重复随机数
	 * 
	 * @param min
	 *            最小值
	 * @param max
	 *            最大值
	 * @param n
	 * @return
	 */
	public static int[] randomCommon(int min, int max, int n) {
		int len = max - min + 1;

		if (max < min || n > len) {
			return new int[0];
		}

		// 初始化给定范围的待选数组
		int[] source = new int[len];
		for (int i = min; i < min + len; i++) {
			source[i - min] = i;
		}

		int[] result = new int[n];
		Random rd = new Random();
		int index = 0;
		for (int i = 0; i < result.length; i++) {
			// 待选数组0到(len-2)随机一个下标
			index = Math.abs(rd.nextInt() % len--);
			// 将随机到的数放入结果集
			result[i] = source[index];
			// 将待选数组中被随机到的数，用待选数组(len-1)下标对应的数替换
			source[index] = source[len];
		}
		return result;
	}

	// 每位加权因子
	private static int power[] = { 7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2 };

	/**
	 * 身份证验证
	 * 
	 * @param idcard
	 *            身份证号
	 * @return
	 */
	public static boolean isValidatedAllIdcard(String idcard) {
		if (idcard.length() == 15) {
			return false;
		}

		return isValidate18Idcard(idcard);
	}

	private static boolean isValidate18Idcard(String idcard) {
		// 非18位为假
		if (idcard.trim().length() != 18) {
			return false;
		}
		// 获取前17位
		String idcard17 = idcard.substring(0, 17);
		// 获取第18位
		String idcard18Code = idcard.substring(17, 18);
		char c[] = null;
		String checkCode = "";
		// 是否都为数字
		if (isDigital(idcard17)) {
			c = idcard17.toCharArray();
		} else {
			return false;
		}

		if (null != c) {
			int bit[] = new int[idcard17.length()];

			bit = converCharToInt(c);
			int sum17 = 0;

			sum17 = getPowerSum(bit);
			// 将和值与11取模得到余数进行校验码判断
			checkCode = getCheckCodeBySum(sum17);
			if (null == checkCode) {
				return false;
			}
			// 将身份证的第18位与算出来的校码进行匹配，不相等就为假
			if (!idcard18Code.equalsIgnoreCase(checkCode)) {
				return false;
			}
		}
		return true;
	}

	private static boolean isDigital(String str) {
		return str == null || "".equals(str) ? false : str.matches("^[0-9]*$");
	}

	private static int[] converCharToInt(char[] c) throws NumberFormatException {
		int[] a = new int[c.length];
		int k = 0;
		for (char temp : c) {
			a[k++] = Integer.parseInt(String.valueOf(temp));
		}
		return a;
	}

	private static int getPowerSum(int[] bit) {
		int sum = 0;
		if (power.length != bit.length) {
			return sum;
		}
		for (int i = 0; i < bit.length; i++) {
			for (int j = 0; j < power.length; j++) {
				if (i == j) {
					sum = sum + bit[i] * power[j];
				}
			}
		}
		return sum;
	}

	private static String getCheckCodeBySum(int sum17) {
		String checkCode = null;
		switch (sum17 % 11) {
		case 10:
			checkCode = "2";
			break;
		case 9:
			checkCode = "3";
			break;
		case 8:
			checkCode = "4";
			break;
		case 7:
			checkCode = "5";
			break;
		case 6:
			checkCode = "6";
			break;
		case 5:
			checkCode = "7";
			break;
		case 4:
			checkCode = "8";
			break;
		case 3:
			checkCode = "9";
			break;
		case 2:
			checkCode = "x";
			break;
		case 1:
			checkCode = "0";
			break;
		case 0:
			checkCode = "1";
			break;
		}
		return checkCode;
	}

	/**
	 * 读取所有HttpServletRequest的 传参键值对
	 * 
	 * @param request
	 */
	public static void PrintfHttpServletRequest(HttpServletRequest request) {
		Enumeration<String> names = request.getParameterNames();
		while (names.hasMoreElements()) {
			String name = names.nextElement();
			Log.i("HttpServletRequest", "Key:" + name + " value:" + request.getParameter(name));
		}
	}
	public static String getHttpServletRequest(HttpServletRequest request){
		String queryString="?";
		Enumeration<String> names = request.getParameterNames();
		while (names.hasMoreElements()) {
			String name = names.nextElement();
			queryString+=name+"="+request.getParameter(name)+"&";
		}
		return queryString.substring(0,queryString.length()-1);
	}
	public static void PrintfSession(HttpServletRequest request) {
		Enumeration<String> names=request.getSession().getAttributeNames();
		while (names.hasMoreElements()) {
			String name = names.nextElement();
			Log.i("Session", "Key:" + name + " value:" + request.getSession().getAttribute(name));
		}
	}
	public static String getRootModsPath(HttpServletRequest request) {
		return new File("").getAbsolutePath() + File.separator+"html"+File.separator+"mods"+File.separator;
	}
	public static String getRootPath(HttpServletRequest request) {
		return new File("").getAbsolutePath() +File.separator;
	}
	public static JSONObject isLogin(HttpServletRequest request,String... args) {
		JSONObject json=new JSONObject();
		String token=request.getParameter("token");
		if(StringIsEmpty(token)||StringIsEmpty(args)||RedisUtlis.get(token).equals("error")){
			json.put("state", request.getAttribute("loginState"));
			json.put("msg", "登录信息失效");
			json.put("code", 101);
			return json;
		}
		return null;
	}
	public static int getUserId(HttpServletRequest request) {
		try {
			String ujson = RedisUtlis.get(getCookie(request,"login"));
			JSONObject reJson=new JSONObject(ujson);
			int uid =  reJson.getInt("uid");
			return uid;
		} catch (Exception e) {
			// TODO: handle exception
		}
	
		return 0;
	}
	public static int getUidOfToken(HttpServletRequest request) {
		try {
			String token=request.getParameter("token");
			if(token==null){
				return -1;
			}
			String ujson = RedisUtlis.get(token);
			if(ujson==null){
				return -1;
			}
			JSONObject reJson=new JSONObject(ujson);
			if(reJson.getString("msg").equals("注册成功")) {
				return 0;
			}
			return reJson.getInt("uid");
		} catch (Exception e) {
			// TODO: handle exception
		}
		return 1;
	}
	public static boolean getUserBen(UUserdata udata) {
		if(udata.getUnbeantime()!=null) {
			if(udata.getUnbeantime().getTime()>System.currentTimeMillis()) {
				return true; //被ban了（带解封时间）
			}
		}
		return false;
	}
	public static void resetCode(HttpServletResponse response) {
		response.setContentType("text/html;charset=UTF-8");
	}

	/**
	 * 创建用户登录token
	 * @return token
	 */
	public static String getToken(HttpServletRequest request, int uid) {
		String address = Tools.getRemoteAddress(request);
		String time = System.currentTimeMillis() + "";
		String token = MD5Signature.sign(address + time + (uid + ""), ServiceStaticValue.SERVICE_URL);
		return token;
	}
}
